#!/bin/ksh93

POSIXLY_CORRECT="true"
export POSIXLY_CORRECT


# If you have no (pd)ksh, use zsh and uncomment next line:
#emulate ksh

# Folding used. Get folding.el for your emacs here:
# http://www.csd.uu.se/~andersl/emacs.shtml
# http://tiny-tools.sourceforge.net/

# To use folding under Vim put this to your ~/.vimrc
# set foldmethod=marker

#############################################################################

# tausta.sh
# Korn Shell script, that changes wallpaper under X Window System
# without conflicting with running XScreenSaver .

# Copyright (C) 2003 Juhapekka Tolvanen and Mika Jorma Saaristo

# This script was originally created by:
# Mika Jorma Saaristo
# http//www.cc.jyu.fi/~mjsaaris/
# mjsaaris (at) cc (dot) jyu (dot) fi

# After that it was heavily modified by:
# Juhapekka "naula" Tolvanen
# juhtolv (at) iki (dot) fi
# http://iki.fi/juhtolv

# A function called "watcher" is based on a Korn Shell-script called
# watch.sh that was based on Perl-script called watch.pl (by Juhapekka
# Tolvanen), that was based on code-snippet in man page of
# xscreensaver-command (by Jamie Zawinski ( http://www.jwz.org/ ))

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#############################################################################

XMESSAGE="$(which gxmessage)" || XMESSAGE="xmessage"

# {{{ Luser-serviceable parts

CONFIGDIR="${HOME}/.tausta.sh.dir/"

if . ${CONFIGDIR}tausta.rc.sh
then
    :
else
    if [ -n "$DISPLAY" ]; then
        $XMESSAGE "${0}: Could not load ${CONFIGDIR}tausta.rc.sh" &
    else
        echo "${0}: Could not load ${CONFIGDIR}tausta.rc.sh" >&2
    fi
    exit 1
fi

# }}}

#############################################################################
# No luser-serviceable parts after this line. (I hope.)
#############################################################################

# {{{ Important functions

my_debug()
{
    if test "$MY_DEBUG" = "1"
    then
        echo "$*" >> "$DEBUG_FILE"
    fi
}

print_error()
{
    if [ -n "$DISPLAY" ]; then
        $XMESSAGE "$*" &
    else
        echo "$*" >&2
    fi
    my_debug "print_error $*"
}

# }}}

# {{{ Environment variables

THISBIN="${0}"
export THISBIN

MUST_SLEEP=true
SLEEPED=0

set +o noclobber

# }}}
# {{{ First status check and saving of PID

STATE="`cat $STATUS 2>/dev/null`"

if test -z "$STATE"
then
    STATE="1"
fi

echo $$ >| $MYPID

# }}}
# {{{ Traps

trap "my_debug 'tausta.ksh.sh: killing signal HUP trapped. exiting...' ; clean_up ; exit 0" HUP
trap "my_debug 'tausta.ksh.sh: killing signal INT trapped. exiting...' ; clean_up ; exit 0" INT
trap "my_debug 'tausta.ksh.sh: killing signal QUIT trapped. exiting...' ; clean_up ; exit 0" QUIT
trap "my_debug 'tausta.ksh.sh: killing signal TERM trapped. exiting...' ; clean_up ; exit 0" TERM

trap "my_debug 'Hurry up loading' ; SLEEPED="$(( ${SLEEPED} + ${CHECK_INTERVAL} + ${INTERVAL} ))" ; export SLEEPED ; MUST_SLEEP=false ; export MUST_SLEEP" USR1
#trap "SLEEPED="$(( ${SLEEPED} + ${CHECK_INTERVAL} + ${INTERVAL} ))" ; export SLEEPED ; MUST_SLEEP=false ; export MUST_SLEEP" USR1
trap "my_debug 'tausta.ksh.sh: USR2 signal trapped. re-execing' ; oh_shit_exec signal USR2" USR2

# }}}
# {{{ Functions

clean_up()
{
    echo $STATE >| $STATUS
    MUST_SLEEP=false
    MUST_SLEEP_F=false
    my_debug "cleaning $MYPID"
    echo "0" >| $MYPID
    my_debug "cleaning $WATCHERFILE"
    echo "0" >| $WATCHERFILE
    if test "$KDESCREENSAVER" == "0"
    then
        my_debug "killing HELPPID $( cat $HELPPIDFILE )"
        kill -15 $( cat $HELPPIDFILE )
        my_debug "killing WATCHERPID $( cat $WATCHERPIDFILE )"
        kill -15 $( cat $WATCHERPIDFILE )
    fi
}

oh_shit()
{
    my_debug 'Oh, Shit! Something is fscked up!'
    clean_up
    exit 1
}

oh_shit_exec()
{
    my_debug "Oh, Shit! I must re-exec myself because of ${1} ${2}"
    clean_up
    my_debug "exec $THISBIN"
    exec $THISBIN
}


check_wm()
{
    if ps -C $WM -o user= | grep $USER > /dev/null 2>&1
    then
        my_debug "Window manager called $WM is still running"
    else
        my_debug "Window manager called $WM is no more running. It seems we are not in X Window System"
        echo $STATE >| $STATUS
        oh_shit
    fi
}

if test ! -f "$IMAGES"
then
    print_error Picturelist not found
    oh_shit
fi

#if test "$KDESCREENSAVER" = "0"
#then
#my_debug "We are not under KDE"
check_watcher()
{
    if ps -o user= $( cat ${WATCHERPIDFILE} ) | grep $USER > /dev/null 2>&1
    then
        my_debug "watcher is still running"
    else
        my_debug "watcher is no more running."
        print_error "watcher is no more running."
        oh_shit_exec watcher died
    fi
}

check_helper()
{
    if ps -o user= $( cat ${HELPPIDFILE} ) | grep $USER > /dev/null 2>&1
    then
        my_debug "xscreensaver-command is still running"
    else
        my_debug "xscreensaver-command is no more running."
        print_error "xscreensaver-command is no more running."
        oh_shit_exec xscreensaver-command died
    fi
}
#fi

check_all()
{
    check_wm
    if test "$KDESCREENSAVER" = "0"
    then
        my_debug "We are not under KDE"
        check_watcher
        check_helper
    fi
}


########################################################################
# THE GREAT WATCHER FUNCTION

# This function tells for tausta.ksh.sh, if xscreensaver
# is running some screen hack.

# Bourne Shell or POSIX Shell can't do "<some command> |&" "read -p"
# but implementations of Korn Shell (ksh88, ksh93 and pdksh and zsh) 
# can. But then I found "mkfifo". Originally this was separate
# script but then it became function. Unfortunately using fifo seems
# really unreliable way for me.


watcher ()
{
    set +o noclobber
    BLANKED="0"
    xscreensaver-command -watch |&
    HELPPID="$!"
    my_debug "HELPPID is $HELPPID"
    echo $HELPPID >| $HELPPIDFILE
    check_helper
    while read -p foo waste
    do
        case "$foo" in
            UNBLANK)
                my_debug "xscreensaver-command: UNBLANK"
                echo "0" >| $WATCHERFILE
                ;;
            BLANK)
                my_debug "xscreensaver-command: BLANK"
                echo "1" >| $WATCHERFILE
                ;;
            LOCK)
                my_debug "xscreensaver-command: LOCK"
                echo "1" >| $WATCHERFILE
                ;;
            RUN)
                my_debug "xscreensaver-command: RUN"
                echo "1" >| $WATCHERFILE
                ;;
            *)
                :
        esac
    done
}

if test $KDESCREENSAVER = "1"
then
    my_debug "We are under KDE"
    set +o noclobber
    BLANKED="0"
    echo "0" >| $WATCHERFILE
else
    watcher &
    WATCHERPID="$!"
    my_debug "WATCHERPID is $WATCHERPID"
    set +o noclobber
    echo $WATCHERPID >| $WATCHERPIDFILE
    check_watcher
fi



########################################################################

check_screensavers()
{
    P=1
    if test "$( cat ${WATCHERFILE} )" = "1"
    then
        my_debug "screensaver found running"
        P=0
    else
        my_debug "no screensaver found running"
        P=1
    fi
    return $P
}

getcmdline()
{
    CMD="`sed '/^$/d;/^#/d' $IMAGES|sed -n ${STATE}p`"
    if [ -z "$CMD" ]; then
        STATE="1";
        CMD="`sed '/^$/d;/^#/d' $IMAGES|sed -n ${STATE}p`"
        if [ -z "$CMD" ]; then
            $XMESSAGE "Can not get next image"
            oh_shit
        fi
    fi
}

# }}}
# {{{ Sleep loop

sleep_loop()
{
    my_debug "we are in the sleep_loop"
    MUST_SLEEP=true
    SLEEPED=0
    while test $MUST_SLEEP = "true"
    do
        my_debug "not yet sleeped enough: SLEEPED=${SLEEPED}"
        if test $SLEEPED -ge $INTERVAL
        then
            my_debug "sleeped enough: SLEEPED=${SLEEPED}"
            MUST_SLEEP=false
        fi
        # NOTE! If you use plain “sleep” here, AT&T KSH93 will use its
        # builtin sleep command and becomes unreliable!: It runs for a
        # while, but if you give signal USR1 few times, it just exits
        # and leaves function called “watcher” and
        # “xscreensaver-command -watch” running as forked processes.
        /bin/sleep $CHECK_INTERVAL
        SLEEPED="$(( ${SLEEPED} + ${CHECK_INTERVAL} ))"
        my_debug "SLEEPED=${SLEEPED}"
        check_all
    done
}

# }}}
# {{{ Misc

check_all
sleep $CHECK_INTERVAL

# }}}
# {{{ Main loop

while true
do
    my_debug "we are in mainloop"
    while check_screensavers
    do
        my_debug "we are in check screensavers-loop"
        my_debug "screensaver is running"
        check_all
        sleep $CHECK_INTERVAL
    done
    my_debug "deciding background-loading command ..."
    getcmdline
    check_all
    echo "$CMD" >| $LASTCOMMANDFILE
    echo "$CMD" | sh -s
    my_debug "loading of background invoked with command $CMD"
    STATE="$(( ${STATE} + 1 ))"
    echo $STATE >| $STATUS
    check_all
    MUST_SLEEP=false
    sleep_loop
done

print_error "Somehow we went out from main loop. Oh, Shit!"
oh_shit

# }}}

